c: hyperndsubs - subroutines needed for hypernd program, afv 1/2000

      subroutine dmzer(a,n)
c:  zeros each of the n elements in real*8 array a
c:
      implicit real*8(a-h,o-z)
      dimension a(n)
      do 10 i=1,n
         a(i)=0.0d0
   10 continue
      return
      end


      subroutine ave(x,n,xave,sdmean)
c: computes average and S.D. of mean for numbers x(1) through x(n)
c    handles very large or very small numbers
c:
      implicit real*8(a-h,o-z)
      dimension x(n)
      xave=x(1)
      sdmean=0.0d0
      if(n.le.1) return
c  find largest x value
      xmax=0.0d0
      do 10 i=1,n
   10 if(abs(x(i)).gt.xmax) xmax=abs(x(i))
      if(xmax.eq.0.0d0) return
      scale=1.0d0/xmax
c
      xsum=0.0d0
      xxsum=0.0d0
      do 50 i=1,n
         xtem=x(i)*scale
         xsum=xsum+xtem
         xxsum=xxsum+xtem*xtem
   50 continue
      dn=float(n)
      xave=xsum/dn
      xave=xave/scale
      std=sqrt((xxsum-xsum*xsum/dn)/(dn-1.0d0))/scale
      sdmean=std/sqrt(dn-1.0d0)
      return
      end


      subroutine bivariate(sig1,sig2,c12,g1,g2)
c returns 2 random numbers from bivariate Gaussian dist. with specified
c  std's and correlation coef.
c  i.e., 
c p(g1,g2)/norm 
c = exp(-(sig2**2*g1**2+sig1**2*g2**2)/(2[sig1**2+sig2**2-(c12*sig1*sig2)**2])
c = exp(-(var2*g1**2+var1*g2**2)/(2(var1+var2-c12**2*var1*var2))
c e.g.,  see p 28 of Wax (Chandrasekhar article, Eq.(178) )
c uses Box-Muller transformation;  see p. 203 of Num. Rec., 1st ed.
c and then covariance formula from App. G of Allen and Tildesley, p. 348.
      implicit real*8(a-h,o-z)
1     v1=2.0d0*prngen(0)-1.0d0   ! pick coords in square (-1,1)x(-1,1)
      v2=2.0d0*prngen(0)-1.0d0
      r=v1**2+v2**2
      if(r.ge.1.0d0) go to 1     ! discard pairs that are not in unit circle
      fac=sqrt(-2.0d0*log(r)/r)  ! Box-Muller transformation
      gset1=v1*fac      
      gset2=v2*fac     

      g1=sig1*gset1
      g2=sig2*(c12*gset1 + sqrt(1.0d0-c12**2)*gset2)
      return
      end  


      subroutine seedit(iseed)
c:  Seeds the random number generator with iseed.
c:  If iseed=0, iseed is set in here and returned for reference.
c:  WARNING - iseed gets changed if iseed=0  - - do not pass a constant integer!!!
c:
      implicit real*8(a-h,o-z)
      common/gasdevcom/iset
      integer time
c
      if(iseed.eq.0) then
c   VAX (vax) version:
c       iseed = 20*iabs(ifix(secnds(0.0)))-1
c/use next1 on i686
        iseed = time()*2 - 1
c/use next1 on allbut i686
c       iseed = time(idum)*2 - 1
      end if                             
      iseed=mod(abs(iseed),100000000)  ! ensures seed is not too large

c  after this first call to prngen, all future calls should pass zero arg.
      iseed=-abs(iseed)
      xdum=prngen(iseed)
      iset=0
      return
      end


      function prngen ( init )
c
c: >>> prngen -- portable random number generator
c
c            Note:  this function is lifted from the routine RAN3
c                   described by Press et. al. in _Numerical Recipes_
c                   I chose this function because of its portability and
c                   because it only requires one floating point multiply
c                   after the initialization call.
c
c     The routine is initialized (or reinitialized) if it is called with
c         a negative value for init.  Unlike the routine in
c         _Numerical Recipes_, the value of init is NOT altered by the
c         function.  Similarly, prngen returns ZERO on an initialization
c         call, and only returns random numbers when init.ge.0.
c
c     implicit none
      real*8 prngen
c ---                            !       avoids trouble
c
      integer   magic
c ---                            !       Knuth's magic number
      parameter (magic=55)
      integer   magoff
c ---                            !       another magic number
      parameter (magoff=31)
      integer   zero
c ---                            !       constant = 0
      parameter (zero=0)
      integer   BIGINT
c ---                            !       any large integer
      parameter (BIGINT=1000000000)
      integer   SEED
c ---                            !       any seed integer (.lt.BIGINT)
      parameter (SEED  = 161803398)
c
      logical   FIRST
c ---                            !       flag for first call
      integer   Next, NextP
c ---                            !       pointers to table slots
      integer   Table(magic)
c ---                            !       table of seed derivatives
      integer   i,j,k,ii
c ---                            !       miscellaneous counters
c
      real*8      scale
c ---                            !       scale factor to get reals
      parameter (scale=1e0/BIGINT)
c
      save FIRST, Table, Next, NextP
c
      data FIRST / .true. /
c
c--------------   start initialization section  here  --------------------
c
      IF ( FIRST .or. init.lt.zero ) THEN
        if(init.gt.SEED) stop 'seed too large for prngen'  

        j = mod(SEED-iabs(init), BIGINT)
        Table(magic) = j
        k = 1
c
        do 10 i=1,magic-1
          ii        = mod(21*i,magic)
          Table(ii) = k
          k         = j - k
          if ( k.lt.zero ) k = k + BIGINT
          j = Table(ii)
   10   continue
c
        do 20 k = 1, 4
          do 30 i = 1, magic
            Table(i) = Table(i) - Table(1+mod(i+magoff-1,magic))
            if ( Table(i).lt.zero ) Table(i) = Table(i) + BIGINT
   30     continue
   20   continue
c
        Next  = 0
        NextP = magoff
c
        IF ( init.lt.zero ) THEN
          prngen = 0.0d0
          FIRST  = .false.
          return
        END IF
c
        FIRST = .false.
      END IF
c
c-------------  here's where we go if we are returning a value  -------
c
      Next  = Next  + 1
      if ( Next  .gt. magic ) Next  = 1
c
      NextP = NextP + 1
      if ( NextP .gt. magic ) NextP = 1
c
      j = Table(Next) - Table(NextP)
      if ( j.lt.zero ) j = j + BIGINT
c
      Table(Next) = j
      prngen = scale*j
c
      return
      end


      function gasdev(idum)
c returns Gaussian-distributed random number, unit variance
c uses Box-Muller transformation;  see p. 203 of Num. Rec., 1st ed.
      implicit real*8(a-h,o-z)
      common/gasdevcom/iset
      save gset

      if (iset.ne.1) then
1       v1=2.0d0*prngen(0)-1.0d0   ! pick coords in square (-1,1)x(-1,1)
        v2=2.0d0*prngen(0)-1.0d0
        r=v1**2+v2**2
        if(r.ge.1.0d0) go to 1     ! discard pairs that are not in unit circle
        fac=sqrt(-2.0d0*log(r)/r)  ! Box-Muller transformation
        gset=v1*fac                ! save this one for next time
        gasdev=v2*fac              ! return this one
        iset=1
      else
        gasdev=gset
        iset=0
      endif
      return
      end  
